/*
 * tree_examples.cpp
 *
 *  Created on: 28.11.2012
 *      Author: trifon
 */

#include "tree.cpp"
#include <cmath>

void simpleTree()
{
	BinTree<> t1,t2,t3;
	t1.createTree(1, t2, t3);
	t1.print();
}

void createFullTree(BinTree<>& t, int n, int l) {
	if (l <= n)
	{
		BinTree<> lt, rt;
		createFullTree(lt, n, l+1);
		createFullTree(rt, n, l+1);
		t.createTree(l, lt, rt);
	}
}

void fullTree(int n)
{
	BinTree<> t;
	createFullTree(t, n, 1);
	t.print();
}

int depth(BinTree<>::I it)
{
	if (!it)
		return 0;
	return 1 + max(depth(++it), depth(it++));
}

void depthTree()
{
	BinTree<> t;
	createFullTree(t, 4, 1);
	cout << depth(t.iterator()) << endl;
}

template <typename T>
bool equal(BinTreeIterator<T> it1, BinTreeIterator<T> it2)
{
	if (!it1 && !it2)
		return true;
	if (!it1 || !it2)
		return false;
	if (*it1 != *it2)
		return false;
	return equal(++it1, ++it2) && equal(it1++, it2++);
}

void equalTrees()
{
	BinTree<> t1, t2;
	createFullTree(t1, 4, 1);
	createFullTree(t2, 3, 1);
	cout << "The trees are";
	if (!equal(t1.iterator(), t2.iterator()))
	{
		cout << " not";
	}
	cout << " equal." << endl;
}

// <expr> ::= 0-9 | (<expr><op><expr>)
// <op>   ::= + | - | * | /

void createExprTree(BinTree<char>& t, char const*& s)
{
	BinTree<char> lt, rt;
	if (*s >= '0' && *s <= '9')
	{
		t.createTree(*s++, lt, rt);
	} else
	{
		// *s == '('
		createExprTree(lt, ++s);
		// *s е операция
		char op = *s;
		createExprTree(rt, ++s);
		// *s == ')'
		s++;
		t.createTree(op, lt, rt);

	}
}

int evaluate(BinTreeIterator<char> it)
{
	if (*it >= '0' && *it <= '9')
		return *it - '0';
	int lv = evaluate(++it);
	int rv = evaluate(it++);
	switch (*it)
	{
	case '+':return lv + rv;
	case '-':return lv - rv;
	case '*':return lv * rv;
	case '/':return lv / rv;
	}
	return 0;
}

void printInfix(BinTreeIterator<char> it)
{
	if (!it)
		return;
	if (*it >= '0' && *it <= '9')
		cout << *it;
	else
	{
		cout << '(';
		printInfix(++it);
		cout << ' ' << *it << ' ';
		printInfix(it++);
		cout << ')';
	}
}

void printPostfix(BinTreeIterator<char> it)
{
	if (!it)
		return;
	if (*it >= '0' && *it <= '9')
		cout << *it;
	else
	{
		printPostfix(++it);
		printPostfix(it++);
		cout << *it;
	}
}

void exprTree()
{
	const int MAX = 100;
	char s[MAX];
	cout << "Expression: ";
	cin.getline(s, MAX);
	BinTree<char> t;
	char const* p = s;
	createExprTree(t, p);
	t.print();
	cout << endl;
//	printInfix(t.iterator());
//	cout << endl;
//	printPostfix(t.iterator());
//	cout << endl;
	cout << evaluate(t.iterator()) << endl;
}

int main()
{
	simpleTree();
	fullTree(4);
	depthTree();
	equalTrees();
	exprTree();

	return 0;
}



